# -*- coding: utf-8 -*-
# Generated by Django 1.11.5 on 2017-09-08 15:50
from __future__ import unicode_literals

import jsonfield.fields
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('aa_stripe', '0011_auto_20170907_1201'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='stripewebhook',
            options={'ordering': ['-created']},
        ),
        migrations.AlterField(
            model_name='stripecoupon',
            name='amount_off',
            field=models.DecimalField(blank=True, decimal_places=2, help_text='Amount (in the currency specified) that will be taken off the subtotal of any invoices for thiscustomer.', max_digits=10, null=True),
        ),
        migrations.AlterField(
            model_name='stripecoupon',
            name='currency',
            field=models.CharField(blank=True, choices=[('usd', 'USD'), ('aed', 'AED'), ('afn', 'AFN'), ('all', 'ALL'), ('amd', 'AMD'), ('ang', 'ANG'), ('aoa', 'AOA'), ('ars', 'ARS'), ('aud', 'AUD'), ('awg', 'AWG'), ('azn', 'AZN'), ('bam', 'BAM'), ('bbd', 'BBD'), ('bdt', 'BDT'), ('bgn', 'BGN'), ('bif', 'BIF'), ('bmd', 'BMD'), ('bnd', 'BND'), ('bob', 'BOB'), ('brl', 'BRL'), ('bsd', 'BSD'), ('bwp', 'BWP'), ('bzd', 'BZD'), ('cad', 'CAD'), ('cdf', 'CDF'), ('chf', 'CHF'), ('clp', 'CLP'), ('cny', 'CNY'), ('cop', 'COP'), ('crc', 'CRC'), ('cve', 'CVE'), ('czk', 'CZK'), ('djf', 'DJF'), ('dkk', 'DKK'), ('dop', 'DOP'), ('dzd', 'DZD'), ('egp', 'EGP'), ('etb', 'ETB'), ('eur', 'EUR'), ('fjd', 'FJD'), ('fkp', 'FKP'), ('gbp', 'GBP'), ('gel', 'GEL'), ('gip', 'GIP'), ('gmd', 'GMD'), ('gnf', 'GNF'), ('gtq', 'GTQ'), ('gyd', 'GYD'), ('hkd', 'HKD'), ('hnl', 'HNL'), ('hrk', 'HRK'), ('htg', 'HTG'), ('huf', 'HUF'), ('idr', 'IDR'), ('ils', 'ILS'), ('inr', 'INR'), ('isk', 'ISK'), ('jmd', 'JMD'), ('jpy', 'JPY'), ('kes', 'KES'), ('kgs', 'KGS'), ('khr', 'KHR'), ('kmf', 'KMF'), ('krw', 'KRW'), ('kyd', 'KYD'), ('kzt', 'KZT'), ('lak', 'LAK'), ('lbp', 'LBP'), ('lkr', 'LKR'), ('lrd', 'LRD'), ('lsl', 'LSL'), ('mad', 'MAD'), ('mdl', 'MDL'), ('mga', 'MGA'), ('mkd', 'MKD'), ('mmk', 'MMK'), ('mnt', 'MNT'), ('mop', 'MOP'), ('mro', 'MRO'), ('mur', 'MUR'), ('mvr', 'MVR'), ('mwk', 'MWK'), ('mxn', 'MXN'), ('myr', 'MYR'), ('mzn', 'MZN'), ('nad', 'NAD'), ('ngn', 'NGN'), ('nio', 'NIO'), ('nok', 'NOK'), ('npr', 'NPR'), ('nzd', 'NZD'), ('pab', 'PAB'), ('pen', 'PEN'), ('pgk', 'PGK'), ('php', 'PHP'), ('pkr', 'PKR'), ('pln', 'PLN'), ('pyg', 'PYG'), ('qar', 'QAR'), ('ron', 'RON'), ('rsd', 'RSD'), ('rub', 'RUB'), ('rwf', 'RWF'), ('sar', 'SAR'), ('sbd', 'SBD'), ('scr', 'SCR'), ('sek', 'SEK'), ('sgd', 'SGD'), ('shp', 'SHP'), ('sll', 'SLL'), ('sos', 'SOS'), ('srd', 'SRD'), ('std', 'STD'), ('svc', 'SVC'), ('szl', 'SZL'), ('thb', 'THB'), ('tjs', 'TJS'), ('top', 'TOP'), ('try', 'TRY'), ('ttd', 'TTD'), ('twd', 'TWD'), ('tzs', 'TZS'), ('uah', 'UAH'), ('ugx', 'UGX'), ('uyu', 'UYU'), ('uzs', 'UZS'), ('vnd', 'VND'), ('vuv', 'VUV'), ('wst', 'WST'), ('xaf', 'XAF'), ('xcd', 'XCD'), ('xof', 'XOF'), ('xpf', 'XPF'), ('yer', 'YER'), ('zar', 'ZAR'), ('zmw', 'ZMW')], help_text='If amount_off has been set, the three-letter ISO code for the currency of the amount to take off.', max_length=3, null=True),
        ),
        migrations.AlterField(
            model_name='stripecoupon',
            name='duration_in_months',
            field=models.PositiveIntegerField(blank=True, help_text='If duration is repeating, the number of months the coupon applies. Null if coupon duration is forever or once.', null=True),
        ),
        migrations.AlterField(
            model_name='stripecoupon',
            name='metadata',
            field=jsonfield.fields.JSONField(default=dict, help_text='Set of key/value pairs that you can attach to an object. It can be useful for storing additional information about the object in a structured format.'),
        ),
        migrations.AlterField(
            model_name='stripecoupon',
            name='percent_off',
            field=models.PositiveIntegerField(blank=True, help_text='Percent that will be taken off the subtotal of any invoicesfor this customer for the duration of the coupon. For example, a coupon with percent_off of 50 will make a $100 invoice $50 instead.', null=True),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.5 on 2017-09-21 21:51
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('aa_stripe', '0013_stripesubscription_at_period_end'),
    ]

    operations = [
        migrations.AddField(
            model_name='stripecharge',
            name='charge_attempt_failed',
            field=models.BooleanField(default=False),
        ),
    ]



# -*- coding: utf-8 -*-
import sys
import traceback
from time import sleep

from django.core.management.base import BaseCommand

from aa_stripe.models import StripeSubscription

try:
    from raven.contrib.django.raven_compat.models import client
except ImportError:
    pass


class Command(BaseCommand):
    """
    Terminates outdated subscriptions at period end.

    Should be run hourly.
    Exceptions are queued and returned at the end.
    """

    help = "Terminate outdated subscriptions"

    def handle(self, *args, **options):
        subscriptions = StripeSubscription.get_subcriptions_for_cancel()
        exceptions = []
        for subscription in subscriptions:
            try:
                subscription.cancel(at_period_end=True)
                sleep(0.25)  # 4 requests per second tops
            except Exception as e:
                exc_type, exc_value, exc_traceback = sys.exc_info()
                try:
                    if client.is_enabled():
                        client.captureException()
                    else:
                        raise
                except NameError:
                    exceptions.append({
                        "obj": subscription,
                        "exc_type": exc_type,
                        "exc_value": exc_value,
                        "exc_traceback": exc_traceback,
                    })

        for e in exceptions:
            print("Exception happened")
            print("Subscription id: {obj.id}".format(obj=e["obj"]))
            traceback.print_exception(e["exc_type"], e["exc_value"], e["exc_traceback"], file=sys.stdout)
        if exceptions:
            sys.exit(1)



# -*- coding: utf-8 -*-
"""
Settings for aa-stripe all start with STRIPE_ namespace. For example your project's "settings.py" file
might look like this:

STRIPE_API_KEY = "api_key"
STRIPE_WEBHOOK_ENDPOINT_SECRET = "secret"

and the settings can be accessed by the aa_stripe.settings.stripe_settings object:

from aa_stripe.settings import stripe_settings
stripe_settings.API_KEY  # "api_key"
"""
from __future__ import unicode_literals

from django.conf import settings
from django.test.signals import setting_changed

DEFAULTS = {
    "PENDING_WEBHOOKS_THRESHOLD": 20,
    "API_KEY": "",
    "WEBHOOK_ENDPOINT_SECRET": "",
    "USER_MODEL": settings.AUTH_USER_MODEL
}


class StripeSettings(object):
    def __getattr__(self, attr):
        if attr not in DEFAULTS:
            raise AttributeError("Invalid API setting: '%s'" % attr)

        val = getattr(settings, "STRIPE_{}".format(attr), DEFAULTS[attr])
        # Cache the result
        setattr(self, attr, val)
        return val


class StripeSettingOutter(object):
    def __init__(self, settings_inner):
        self.settings_inner = settings_inner

    def __getattr__(self, attr):
        return getattr(self.settings_inner, attr)


stripe_settings = StripeSettingOutter(StripeSettings())


def reload_api_settings(*args, **kwargs):
    global stripe_settings
    if kwargs["setting"].startswith("STRIPE_"):
        stripe_settings.settings_inner = StripeSettings()


setting_changed.connect(reload_api_settings)



# -*- coding: utf-8 -*-
# Generated by Django 1.11.5 on 2017-09-25 12:38
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('aa_stripe', '0014_stripecharge_charge_attempt_failed'),
    ]

    operations = [
        migrations.AlterField(
            model_name='stripesubscriptionplan',
            name='amount',
            field=models.BigIntegerField(help_text='In cents. More: https://stripe.com/docs/api#create_plan-amount'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.4 on 2017-09-07 16:01
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('aa_stripe', '0010_auto_20170822_1004'),
    ]

    operations = [
        migrations.AddField(
            model_name='stripewebhook',
            name='parse_error',
            field=models.TextField(blank=True),
        ),
    ]



# -*- coding: utf-8 -*-
import stripe
from django.conf import settings
from django.contrib.sites.models import Site
from django.core.mail import mail_admins
from django.core.management.base import BaseCommand
from django.utils.timezone import now

from aa_stripe.models import StripeWebhook
from aa_stripe.settings import stripe_settings


class StripePendingWebooksLimitExceeded(Exception):
    def __init__(self, pending_webhooks, site):
        self.message = "Pending webhooks threshold limit exceeded, current threshold is {}".format(
            stripe_settings.PENDING_WEBHOOKS_THRESHOLD)
        # send email to admins
        server_env = getattr(settings, "ENV_PREFIX", None)
        email_message = "Pending webhooks for {domain} at {now}:\n\n{webhooks}".format(
            domain=site.domain, now=now(), webhooks="\n".join(webhook["id"] for webhook in pending_webhooks)
        )
        if server_env:
            email_message += "\n\nServer environment: {}".format(server_env)

        mail_admins("Stripe webhooks pending threshold exceeded", email_message)
        super(StripePendingWebooksLimitExceeded, self).__init__(self.message)


class Command(BaseCommand):
    help = "Check pending webhooks at Stripe API"

    def add_arguments(self, parser):
        parser.add_argument(
            "--site",
            help="Site id to use while running the command. First site in the database will be used if not provided."
        )

    def handle(self, *args, **options):
        stripe.api_key = stripe_settings.API_KEY

        site_id = options.get("site")
        site = Site.objects.get(pk=site_id) if site_id else Site.objects.all()[0]
        pending_webhooks = []
        last_event = StripeWebhook.objects.first()
        last_event_id = last_event.id if last_event else None
        try:
            if last_event:
                stripe.Event.retrieve(last_event_id)
        except stripe.error.InvalidRequestError:
            last_event_id = None

        while True:
            event_list = stripe.Event.list(ending_before=last_event_id, limit=100)  # 100 is the maximum
            pending_webhooks += event_list["data"]

            if len(pending_webhooks) > stripe_settings.PENDING_WEBHOOKS_THRESHOLD:
                raise StripePendingWebooksLimitExceeded(pending_webhooks, site)

            if not event_list["has_more"]:
                break
            else:
                last_event_id = event_list["data"][-1]["id"]
